/***********************************************************************************************************
*                                                                                                          *
*     This source code is written by Hao Chen. (2007)                                                      *
*     Department of Biology, Purdue University                                                             *
*     West Lafayette, IN, 47907, USA.                                                                      *
*     Email: chen177@purdue.edu                                                                            *
*     Please cite our paper: Estimating Quality of Template-Based Protein Models                           *
*     by Alignment Stability. Hao Chen & Daisuke Kihara.                                                   *
*     All rights reserved.                                                                                 *
*     The C++ source for SPAD threading program. To compile, use g++ under linux.                          *
*                                                                                                          *
***********************************************************************************************************/
#include <sstream>
#include <fstream>
#include <iostream>
#include <string>
#include <cmath>
#include <valarray>

using namespace std;

const double OPEN_PENALTY=-4.71;  //The open-gap penalty 
const double AFFINE_PENALTY=-0.37;  //The affine penalty

//The directions in the matrix

const int DIAGONAL=1;
const int UP=2;
const int LEFT=3;

//The numbers of the pick-up points in the matrix.

const int THRESHOLD=1000;
const int MAXLENGTH=2000;

//The offset for path-evaluation.

const int OFFSET=3;

//The definitions for the secondary structure.

const int ALPHA_HELIX=0;
const int BETA_SHEET=2;
const int COIL=1;

struct addmatrix
{
		int x;
		int y;
		double score;
		addmatrix()
		{
			x=0;
			y=0;
			score=0;
		}
};
struct outputalignment
{
	double pointscore;
	int x;
	int y;
	string SeqA;
	string SeqB;
};
struct pathmatrix
{
	int right;
	int diagonal;
	int down;
};
string SeqA, SeqB;
struct realalignment
{
	int x;
	int y;
}RealAlignment[2*MAXLENGTH];
int SeqALength, SeqBLength, RealALength=0;
int SeqAStructure[MAXLENGTH],SeqBStructure[MAXLENGTH];
float DisSerie[2*MAXLENGTH],DisSerie1[2*MAXLENGTH];
//char * name;
double fa1[20][MAXLENGTH],fa2[20][MAXLENGTH],Nm1[20][MAXLENGTH],Nm2[20][MAXLENGTH],qf1[20][MAXLENGTH],qf2[20][MAXLENGTH];
int ln1=0,ln2=0;
int plot_flag=0;
double avg=0,dev=0,savg=0,sdev=0;
string ReverseString(string a);

//The definition of functions.

double GetScore(int a, int b, string SqA, string SqB);
int GetPosition(char Base);
void ReadInput(int argc, char* argv[]);
void ReadMatrix(char* MatrixName);
void ReadSecondaryStructure(char* Structure,int* StoreString);
void ReadQuerySecondaryStructure(char* Structure,int* StoreString);
void ReadTemplateSecondaryStructure(char* Structure,int* StoreString);
string ReadSequenceFile(char* TemplateSequence);
void ReadRealAlignment(char* RealAlignment, struct realalignment* StoreString);
int ReadProfile(char * Profile, double fa[20][MAXLENGTH], double Nm[20][MAXLENGTH], double qf[20][MAXLENGTH]);
float Distance(struct realalignment* StoreString, struct outputalignment Pred);
float Distance1(struct realalignment* StoreString, struct outputalignment Pred);
void ReadSequence();
void SubOptimalDP();
void Output();
int Compare(const void*p1, const void*p2);
void ReverseProfile(double a[20][MAXLENGTH],int length);
void ReverseSS(int *a,int length);
double GetScore1(int a, int b, string SqA, string SqB);
double GetScore2(int a, int b, string SqA, string SqB);
//double GetSSScore(int a, int b, string SqA, string SqB);
// The score matrix. 

double SSMatrix[3][3]=
{{1.38,-1.86,-3.83},
 {-1.19,0.81,-0.70},
 {-3.40,-1.21,1.54}};

double ScoreMatrix[20][20];
/*  {{6,-2,-2,-3,-1,-1,-1,0,-2,-2,-2,-1,-1,-3,-1,2,0,-4,-3,0},
   {-2,8,-1,-2,-5,1,0,-3,0,-4,-3,3,-2,-4,-3,-1,-2,-4,-3,-4},
   {-2,-1,8,2,-4,0,0,-1,1,-5,-5,0,-3,-4,-3,1,0,-6,-3,-4},
   {-3,-2,2,9,-5,0,2,-2,-2,-5,-5,-1,-5,-5,-2,0,-2,-6,-5,-5},
   {-1,-5,-4,-5,13,-4,-5,-4,-4,-2,-2,-5,-2,-4,-4,-1,-1,-3,-4,-1},
   {-1,1,0,0,-4,8,3,-3,1,-4,-3,2,-1,-5,-2,0,-1,-3,-2,-3},
   {-1,0,0,2,-5,3,7,-3,0,-5,-4,1,-3,-5,-2,0,-1,-4,-3,-4},
   {0,-3,-1,-2,4,-3,-3,8,-3,-6,-5,-2,-4,-5,-3,0,-2,-4,-5,-5},
   {-2,0,1,-2,-4,1,0,-3,11,-5,-4,-1,-2,-2,-3,-1,-3,-4,3,-5},
   {-2,-4,-5,-5,-2,-4,-5,-6,-5,6,2,-4,2,0,-4,-4,-1,-4,-2,4},
   {-2,-3,-5,-5,-2,-3,-4,-5,-4,2,6,-4,3,1,-4,-4,-2,-2,-2,1},
   {1,3,0,-1,-5,2,1,-2,-1,-4,-4,7,2,-5,-2,0,-1,-4,-3,-3},
   {-1,-2,-3,-5,-2,-1,-3,-4,-2,2,3,-2,8,0,-4,-2,-1,-2,-1,-1},
   {-3,-4,-4,-5,-4,-5,-5,-5,-2,0,1,-5,0,9,-5,-4,-3,1,4,-1},
   {-1,-3,-3,-2,-4,-2,-2,-3,-3,-4,-4,-2,-4,-5,11,-1,-2,-5,-4,-4},
   {2,-1,1,0,-1,0,0,0,-1,-4,-4,0,-2,-4,-1,6,2,-3,-4,-2},
   {0,-2,0,-2,-1,-1,-1,-2,-3,-1,-2,-1,-1,-3,-2,2,7,-4,-2,0},
   {-4,-4,-6,-6,-3,-3,-4,-4,-4,-4,-2,-4,-2,1,-5,-4,-4,16,3,-4},
   {-3,-3,-3,-5,-4,-2,-3,-5,3,-2,-2,-3,-1,4,-4,-3,-2,3,10,-2},
   {0,-4,-4,-5,-1,-3,-4,-5,-5,4,1,-3,1,-1,-4,-2,0,-4,-2,6}
  };*/

int threshold;
valarray<outputalignment> OutputAlignment, OutputAlignment1;
  
int main(int argc, char* argv[])
{
  
  ReadInput(argc, argv);
  //ReadSequence();
  //if((ln1+ln2)*3>ln1*ln2*0.1)
  //{
  //	threshold=(ln1+ln2)*3;
  //}
  //else
  //{
  	threshold=ln1*ln2/10;
  //}
  //extern struct outputalignment OutputAlignment[threshold], OutputAlignment1[threshold];
  OutputAlignment.resize(threshold);
  OutputAlignment1.resize(threshold);
  SubOptimalDP();
  Output();
  return 0;
}

void ReadInput(int argc, char* argv[])
{
	for(int i=0;i<argc;i++)
	{
		if(strcmp(argv[i],"-m")==0)
			ReadMatrix(argv[i+1]);
		if(strcmp(argv[i],"-q")==0)
			SeqA=ReadSequenceFile(argv[i+1]);
		if(strcmp(argv[i],"-t")==0)
			SeqB=ReadSequenceFile(argv[i+1]);
	}
//	SeqA="ILS";
//	SeqB="VS";
	SeqALength=SeqA.length();
	SeqBLength=SeqB.length();
	for(int i=0;i<argc;i++)
	{
		if(strcmp(argv[i],"-sq")==0)
			ReadQuerySecondaryStructure(argv[i+1],SeqAStructure);
		if(strcmp(argv[i],"-st")==0)
			ReadTemplateSecondaryStructure(argv[i+1],SeqBStructure);
		if(strcmp(argv[i],"-r")==0)
			ReadRealAlignment(argv[i+1],RealAlignment);
		if(strcmp(argv[i],"-pq")==0)
		{
			int ln=ReadProfile(argv[i+1],fa1,Nm1,qf1);
			if(ln!=SeqA.length())
			{
				cout<<"The lengths of query sequence and profile "<<argv[i+1]<<" do not match!";
				exit(2);
			}
			ln1=ln;
		}
		if(strcmp(argv[i],"-pt")==0)
		{	
			int ln=ReadProfile(argv[i+1],fa2,Nm2,qf2);
			if(ln!=SeqB.length())
			{
				cout<<"The lengths of template sequence and profile "<<argv[i+1]<<" do not match!";
				exit(2);
			}
			ln2=ln;
		}
		if(strcmp(argv[i],"-plot")==0)
		{
			plot_flag=1;
		}
/*		if(strcmp(argv[i],"-n")==0)
		{
			name=argv[i+1];
		}*/
	}
}

void ReadSequence()
{
  cout << "Please input the first sequence you want to align:\n";
  cin >> SeqA;
  cout << "Please input the second sequence you want to align:\n";
  cin >> SeqB;
  SeqALength=SeqA.length();
  SeqBLength=SeqB.length();
}

void SubOptimalDP()
{
  double *Match=new double[(SeqBLength+1)*(SeqALength+1)];
  double *InsertionX=new double[(SeqBLength+1)*(SeqALength+1)];
  double *InsertionY=new double[(SeqBLength+1)*(SeqALength+1)];
  int *DirectionMatrix=new int[(SeqBLength+1)*(SeqALength+1)];
  double *rMatch=new double[(SeqBLength+1)*(SeqALength+1)];
  double *rInsertionX=new double[(SeqBLength+1)*(SeqALength+1)];
  double *rInsertionY=new double[(SeqBLength+1)*(SeqALength+1)];
  int *rDirectionMatrix=new int[(SeqBLength+1)*(SeqALength+1)];
  double *score=new double[SeqBLength*SeqALength], *sscore=new double[SeqBLength*SeqALength];
  for(int i=0;i<=SeqBLength;i++)
  {
  	for(int  j=0;j<=SeqALength;j++)
	{
		Match[i*(SeqALength+1)+j]=0;
		InsertionX[i*(SeqALength+1)+j]=0;
		InsertionY[i*(SeqALength+1)+j]=0;
		DirectionMatrix[i*(SeqALength+1)+j]=0;
		rMatch[i*(SeqALength+1)+j]=0;
		rInsertionX[i*(SeqALength+1)+j]=0;
		rInsertionY[i*(SeqALength+1)+j]=0;
		rDirectionMatrix[i*(SeqALength+1)+j]=0;
	}
  }
  for(int i=1;i<=SeqALength;i++)
  {
  	//Match[0][i]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	//InsertionX[0][i]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	//InsertionY[0][i]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	DirectionMatrix[i]=LEFT;
	//rMatch[0][i]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	//rInsertionX[0][i]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	//rInsertionY[0][i]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	rDirectionMatrix[i]=LEFT;
  }
  for(int i=1;i<=SeqBLength;i++)
  {
  	//Match[i][0]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	//InsertionX[i][0]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	//InsertionY[i][0]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	DirectionMatrix[i*(SeqALength+1)]=UP;
	//rMatch[i][0]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	//rInsertionX[i][0]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	//rInsertionY[i][0]=OPEN_PENALTY+(i-1)*AFFINE_PENALTY;
	rDirectionMatrix[i*(SeqALength+1)]=UP;
  }
  int numer=0;
  for(int i=1;i<=SeqBLength;i++)
  {
	for(int j=1;j<=SeqALength;j++)
	{
		score[numer]=GetScore1(i,j,SeqA,SeqB);
		sscore[numer]=SSMatrix[SeqAStructure[j-1]][SeqBStructure[i-1]];
		avg+=score[numer];
		savg+=sscore[numer];
		numer++;
	}
  }
  avg=avg/(SeqBLength*SeqALength);
  savg=savg/(SeqBLength*SeqALength);
  for(int i=0;i<(SeqBLength*SeqALength);i++)
  {
  	dev+=(score[i]-avg)*(score[i]-avg);
	sdev+=(sscore[i]-savg)*(sscore[i]-savg);
  }
  dev=sqrt(dev/(SeqBLength*SeqALength));
  sdev=sqrt(sdev/(SeqBLength*SeqALength));
  for(int n=2;n<=(SeqALength+SeqBLength);n++)
  {
	for(int i=1;((i<=SeqBLength)&&(i<n));i++)
	{
		if((n-i)<=SeqALength)
		{
			double temp=0;
			if((Match[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,SeqA,SeqB))>=(InsertionX[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,SeqA,SeqB)))
				temp=Match[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,SeqA,SeqB);
			else
				temp=InsertionX[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,SeqA,SeqB);
			if(temp>=(InsertionY[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,SeqA,SeqB)))
				;
			else
				temp=InsertionY[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,SeqA,SeqB);
			Match[i*(SeqALength+1)+n-i]=temp;
			if((n-i-1)==0)
				InsertionX[i*(SeqALength+1)+n-i]=-1000000;
			else
			{
				if(i!=SeqBLength)
				{
					if((Match[i*(SeqALength+1)+n-i-1]+OPEN_PENALTY)>=(InsertionX[i*(SeqALength+1)+n-i-1]+AFFINE_PENALTY))
						InsertionX[i*(SeqALength+1)+n-i]=Match[i*(SeqALength+1)+n-i-1]+OPEN_PENALTY;
					else
						InsertionX[i*(SeqALength+1)+n-i]=InsertionX[i*(SeqALength+1)+n-i-1]+AFFINE_PENALTY;
				}
				else
				{
					if(Match[i*(SeqALength+1)+n-i-1]>=InsertionX[i*(SeqALength+1)+n-i-1])
						InsertionX[i*(SeqALength+1)+n-i]=Match[i*(SeqALength+1)+n-i-1];
					else
						InsertionX[i*(SeqALength+1)+n-i]=InsertionX[i*(SeqALength+1)+n-i-1];
				}
			}
			if((i-1)==0)
				InsertionY[i*(SeqALength+1)+n-i]=-1000000;
			else
			{
				if((n-i)!=SeqALength)
				{
					if((Match[(i-1)*(SeqALength+1)+n-i]+OPEN_PENALTY)>=(InsertionY[(i-1)*(SeqALength+1)+n-i]+AFFINE_PENALTY))
						InsertionY[i*(SeqALength+1)+n-i]=Match[(i-1)*(SeqALength+1)+n-i]+OPEN_PENALTY;
					else
						InsertionY[i*(SeqALength+1)+n-i]=InsertionY[(i-1)*(SeqALength+1)+n-i]+AFFINE_PENALTY;
				}
				else
				{
					if(Match[(i-1)*(SeqALength+1)+n-i]>=InsertionY[(i-1)*(SeqALength+1)+n-i])
						InsertionY[i*(SeqALength+1)+n-i]=Match[(i-1)*(SeqALength+1)+n-i];
					else
						InsertionY[i*(SeqALength+1)+n-i]=InsertionY[(i-1)*(SeqALength+1)+n-i];
				}
			}
			if(Match[i*(SeqALength+1)+n-i]>=InsertionX[i*(SeqALength+1)+n-i])
			{
				if(Match[i*(SeqALength+1)+n-i]>=InsertionY[i*(SeqALength+1)+n-i])
					DirectionMatrix[i*(SeqALength+1)+n-i]=DIAGONAL;
		  		else
					DirectionMatrix[i*(SeqALength+1)+n-i]=UP;
		  	}
			else
			{
				if(InsertionX[i*(SeqALength+1)+n-i]>=InsertionY[i*(SeqALength+1)+n-i])
					DirectionMatrix[i*(SeqALength+1)+n-i]=LEFT;
				else
					DirectionMatrix[i*(SeqALength+1)+n-i]=UP;
			}
		}
	}
  }
  string rSeqA=ReverseString(SeqA);
  string rSeqB=ReverseString(SeqB);
  ReverseProfile(fa1,ln1);
  ReverseProfile(Nm1,ln1);
  ReverseProfile(qf1,ln1);
  ReverseProfile(fa2,ln2);
  ReverseProfile(Nm2,ln2);
  ReverseProfile(qf2,ln2);
  ReverseSS(SeqAStructure, SeqALength);
  ReverseSS(SeqBStructure, SeqBLength);
/*  {
  	double temp[20][1000];
	for(int i=0;i<20;i++)
	{
		for(int j=0;j<1000;j++)
		{
			temp[i][j]=0;
		}
	}
	for(int i=0;i<ln1;i++)
	{
		for(int j=0;j<20;j++)
		{
			temp[j][i]=fa1[j][ln1-1-i];
		}
	}
	fa1[20][1000]=temp[20][1000];
	for(int i=0;i<20;i++)
	{
		for(int j=0;j<1000;j++)
		{
			temp[i][j]=0;
		}
	}
	for(int i=0;i<ln1;i++)
	{
		for(int j=0;j<20;j++)
		{
			temp[j][i]=Nm1[j][ln1-1-i];
		}
	}
	Nm1=temp;
	for(int i=0;i<20;i++)
	{
		for(int j=0;j<1000;j++)
		{
			temp[i][j]=0;
		}
	}
	for(int i=0;i<ln2;i++)
	{
		for(int j=0;j<20;j++)
		{
			temp[j][i]=fa2[j][ln2-1-i];
		}
	}
	fa2=temp;
	for(int i=0;i<20;i++)
	{
		for(int j=0;j<1000;j++)
		{
			temp[i][j]=0;
		}
	}
	for(int i=0;i<ln2;i++)
	{
		for(int j=0;j<20;j++)
		{
			temp[j][i]=Nm2[j][ln2-1-i];
		}
	}
	Nm2=temp;
  }*/
  for(int n=2;n<=(SeqALength+SeqBLength);n++)
  {
	for(int i=1;((i<=SeqBLength)&&(i<n));i++)
	{
		if((n-i)<=SeqALength)
		{
			double temp=0;
			if((rMatch[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,rSeqA,rSeqB))>=(rInsertionX[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,rSeqA,rSeqB)))
				temp=rMatch[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,rSeqA,rSeqB);
			else
				temp=rInsertionX[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,rSeqA,rSeqB);
			if(temp>=(rInsertionY[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,rSeqA,rSeqB)))
				;
			else
				temp=rInsertionY[(i-1)*(SeqALength+1)+n-i-1]+GetScore2(i,n-i,rSeqA,rSeqB);
			rMatch[i*(SeqALength+1)+n-i]=temp;
			if((n-i-1)==0)
				rInsertionX[i*(SeqALength+1)+n-i]=-1000000;
			else
			{
				if(i!=SeqBLength)
				{
					if((rMatch[i*(SeqALength+1)+n-i-1]+OPEN_PENALTY)>=(rInsertionX[i*(SeqALength+1)+n-i-1]+AFFINE_PENALTY))
						rInsertionX[i*(SeqALength+1)+n-i]=rMatch[i*(SeqALength+1)+n-i-1]+OPEN_PENALTY;
					else
						rInsertionX[i*(SeqALength+1)+n-i]=rInsertionX[i*(SeqALength+1)+n-i-1]+AFFINE_PENALTY;
				}
				else
				{
					if(rMatch[i*(SeqALength+1)+n-i-1]>=rInsertionX[i*(SeqALength+1)+n-i-1])
						rInsertionX[i*(SeqALength+1)+n-i]=rMatch[i*(SeqALength+1)+n-i-1];
					else
						rInsertionX[i*(SeqALength+1)+n-i]=rInsertionX[i*(SeqALength+1)+n-i-1];
				}
			}
			if((i-1)==0)
				rInsertionY[i*(SeqALength+1)+n-i]=-1000000;
			else
			{
				if((n-i)!=SeqALength)
				{
					if((rMatch[(i-1)*(SeqALength+1)+n-i]+OPEN_PENALTY)>=(rInsertionY[(i-1)*(SeqALength+1)+n-i]+AFFINE_PENALTY))
						rInsertionY[i*(SeqALength+1)+n-i]=rMatch[(i-1)*(SeqALength+1)+n-i]+OPEN_PENALTY;
					else
						rInsertionY[i*(SeqALength+1)+n-i]=rInsertionY[(i-1)*(SeqALength+1)+n-i]+AFFINE_PENALTY;
				}
				else
				{
					if(rMatch[(i-1)*(SeqALength+1)+n-i]>=rInsertionY[(i-1)*(SeqALength+1)+n-i])
						rInsertionY[i*(SeqALength+1)+n-i]=rMatch[(i-1)*(SeqALength+1)+n-i];
					else
						rInsertionY[i*(SeqALength+1)+n-i]=rInsertionY[(i-1)*(SeqALength+1)+n-i];
				}
			}
			if(rMatch[i*(SeqALength+1)+n-i]>=rInsertionX[i*(SeqALength+1)+n-i])
			{
				if(rMatch[i*(SeqALength+1)+n-i]>=rInsertionY[i*(SeqALength+1)+n-i])
					rDirectionMatrix[i*(SeqALength+1)+n-i]=DIAGONAL;
		  		else
					rDirectionMatrix[i*(SeqALength+1)+n-i]=UP;
		  	}
			else
			{
				if(rInsertionX[i*(SeqALength+1)+n-i]>=rInsertionY[i*(SeqALength+1)+n-i])
					rDirectionMatrix[i*(SeqALength+1)+n-i]=LEFT;
				else
					rDirectionMatrix[i*(SeqALength+1)+n-i]=UP;
			}
		}
	}
  }
	struct addmatrix *AddMatrix = new struct addmatrix[SeqBLength*SeqALength];
	int m=sizeof(AddMatrix[SeqBLength*SeqALength]);
	for(int i=0;i<SeqBLength;i++)
	{
		for(int j=0;j<SeqALength;j++)
		{
			AddMatrix[i*SeqALength+j].x=j;
			AddMatrix[i*SeqALength+j].y=i;
			AddMatrix[i*SeqALength+j].score=Match[(i+1)*(SeqALength+1)+j+1]+rMatch[(SeqBLength-i)*(SeqALength+1)+SeqALength-j]-GetScore2(SeqBLength-i,SeqALength-j,rSeqA,rSeqB);
		}
	}
//	qsort(AddMatrix,(SeqBLength*SeqALength),sizeof(addmatrix), Compare);
	for(int i=1;i<(SeqALength*SeqBLength);i++)
	{
		struct addmatrix temp;
		int j=i;
		while(j!=0)
		{
			if(AddMatrix[j].score>AddMatrix[j-1].score)
			{
				temp=AddMatrix[j-1];
				AddMatrix[j-1]=AddMatrix[j];
				AddMatrix[j]=temp;
				j--;

			}
			else
				break;
		}
	}
	//threshold = OutputAlignment.size();
	for(int i=0;i<threshold;i++)
	{
		int x=AddMatrix[i].x;
		int y=AddMatrix[i].y;
		OutputAlignment[i].pointscore=AddMatrix[i].score;
		OutputAlignment[i].x=AddMatrix[i].x;
		OutputAlignment[i].y=AddMatrix[i].y;
		OutputAlignment[i].SeqA+=SeqA[AddMatrix[i].x];
		OutputAlignment[i].SeqB+=SeqB[AddMatrix[i].y];
		while((x!=0)||(y!=0))
		{
			if(DirectionMatrix[y*(SeqALength+1)+x]==DIAGONAL)
			{
				OutputAlignment[i].SeqA=SeqA[x-1]+OutputAlignment[i].SeqA;
				OutputAlignment[i].SeqB=SeqB[y-1]+OutputAlignment[i].SeqB;
				x--;
				y--;
			}
			if(DirectionMatrix[y*(SeqALength+1)+x]==UP)
			{
				OutputAlignment[i].SeqA="-"+OutputAlignment[i].SeqA;
				OutputAlignment[i].SeqB=SeqB[y-1]+OutputAlignment[i].SeqB;
				y--;
			}
			if(DirectionMatrix[y*(SeqALength+1)+x]==LEFT)
			{
				OutputAlignment[i].SeqA=SeqA[x-1]+OutputAlignment[i].SeqA;
				OutputAlignment[i].SeqB="-"+OutputAlignment[i].SeqB;
				x--;
			}
		}
		x=SeqALength-AddMatrix[i].x-1;
		y=SeqBLength-AddMatrix[i].y-1;
		while((x!=0)||(y!=0))
		{
			if(rDirectionMatrix[y*(SeqALength+1)+x]==DIAGONAL)
			{
				OutputAlignment[i].SeqA=OutputAlignment[i].SeqA+SeqA[SeqALength-x];
				OutputAlignment[i].SeqB=OutputAlignment[i].SeqB+SeqB[SeqBLength-y];
				x--;
				y--;
			}
			if(rDirectionMatrix[y*(SeqALength+1)+x]==UP)
			{
				OutputAlignment[i].SeqA=OutputAlignment[i].SeqA+"-";
				OutputAlignment[i].SeqB=OutputAlignment[i].SeqB+SeqB[SeqBLength-y];
				y--;
			}
			if(rDirectionMatrix[y*(SeqALength+1)+x]==LEFT)
			{
				OutputAlignment[i].SeqA=OutputAlignment[i].SeqA+SeqA[SeqALength-x];
				OutputAlignment[i].SeqB=OutputAlignment[i].SeqB+"-";
				x--;
			}
		}
	}
	delete[] Match;
	delete[] InsertionX;
	delete[] InsertionY;
	delete[] DirectionMatrix;
	delete[] rMatch;
	delete[] rInsertionX;
	delete[] rInsertionY;
	delete[] rDirectionMatrix;
	delete[] score, sscore;
}

void Output()
{
	int j=0;
	//threshold = OutputAlignment.size();
	for(int i=0;i<threshold;i++)
	{
		if((i!=0)&&(OutputAlignment[i].SeqA==OutputAlignment1[j-1].SeqA)&&(OutputAlignment[i].SeqB==OutputAlignment1[j-1].SeqB))
		;
		else
		{
			OutputAlignment1[j]=OutputAlignment[i];
		j++;
		}
	}
/*	char filename[100]="";
	strcat(filename, "output1-");
	strcat(filename, name);
	strcat(filename, ".dat");*/
	ofstream out("output1.dat");
	cout<<"We get "<<j<<" suboptimal pathways.\n";
	out<<"We get "<<j<<" suboptimal pathways.\n";
	for(int i=0;i<j;i++)
	{
		/*cout<<"The No."<<i+1<<" point score is "<<OutputAlignment1[i].pointscore<<".\n";
		cout<<"The position of this point in the matrix should be x="<<OutputAlignment1[i].x<<", y="<<OutputAlignment1[i].y<<".\n";
		cout<<"The best alignment going through this point is:\n";
		cout<<("A:"+OutputAlignment1[i].SeqA+".\n");
		cout<<("B:"+OutputAlignment1[i].SeqB+".\n\n");*/
		out<<"The No."<<i+1<<" point score is "<<OutputAlignment1[i].pointscore<<".\n";
		out<<"The position of this point in the matrix should be x="<<OutputAlignment1[i].x<<", y="<<OutputAlignment1[i].y<<".\n";
		out<<"The best alignment going through this point is:\n";
		out<<("A:"+OutputAlignment1[i].SeqA+".\n");
		out<<("B:"+OutputAlignment1[i].SeqB+".\n\n");
	}
	out.close();
/*	strcpy(filename, "");
	strcat(filename, "pathmatrix-");
	strcat(filename, name);
	strcat(filename, ".dat");*/
	ofstream out1("pathmatrix.dat");
	struct pathmatrix PathMatrix[SeqBLength+1][SeqALength+1];
	for(int p=0;p<=SeqBLength;p++)
	{
		for(int q=0;q<=SeqALength;q++)
		{
			PathMatrix[p][q].right=0;
			PathMatrix[p][q].diagonal=0;
			PathMatrix[p][q].down=0;
		}
	}
	for(int i=0;(i<j)&&(i<(SeqALength*SeqBLength));i++)
	{
		int x=0,y=0;
		PathMatrix[y][x].diagonal+=1;
		for(int k=0;k<OutputAlignment1[i].SeqA.length();k++)
		{
			if(OutputAlignment1[i].SeqA[k]=='-')
			{
				y++;
				PathMatrix[y][x].down+=1;
			}
			else
			{
				if(OutputAlignment1[i].SeqB[k]=='-')
				{
					x++;
					PathMatrix[y][x].right+=1;
				}
				else
				{
					x++;
					y++;
					PathMatrix[y][x].diagonal+=1;
				}
			}
		}
	}
	out1<<"      -";
	for(int i=0;i<SeqALength;i++)
	{
		out1<<"   "<<SeqA[i];
	}
	out1<<"\n";
	string temp="-"+SeqB;
	for(int i=0;i<(SeqBLength+1);i++)
	{
		out1<<"  "<<temp[i];
		for(int k=0;k<(SeqALength+1);k++)
		{
			out1<<"  "<<PathMatrix[i][k].right<<"-"<<PathMatrix[i][k].diagonal<<"-"<<PathMatrix[i][k].down;
		}
		out1<<"\n";
	}
/*	for(int i=0;i<(SeqBLength+1);i++)
	{
		for(int k=0;k<(SeqALength+1);k++)
		{
			cout<<PathMatrix[i][k].right<<" "<<PathMatrix[i][k].diagonal<<" "<<PathMatrix[i][k].down<<"\t"<<i<<"\t"<<k<<endl;
		}
	}*/
	out1.close();
	ofstream out2("confidence.dat");
	int x=0,y=0,k=0;
	for(int i=0;i<OutputAlignment1[0].SeqA.length();i++)
	{
		double sum, conf;
		if(OutputAlignment1[0].SeqA[i]=='-')
		{
			y++;
			sum=0;
			for(k=0;k<(SeqALength+1);k++)
			{
				sum+=abs(x-k)*PathMatrix[y][k].down;
				sum+=abs(x-k)*PathMatrix[y][k].diagonal;
			}
			conf=sum/((double)j);
		}
		else
		{
			if(OutputAlignment1[0].SeqB[i]=='-')
			{
				x++;
				sum=0;
				for(k=0;k<(SeqBLength+1);k++)
				{
					sum+=abs(y-k)*PathMatrix[k][x].right;
					sum+=abs(y-k)*PathMatrix[k][x].diagonal;
				}
				conf=sum/((double)j);
			}
			else
			{
				x++;
				y++;
				sum=0;
				for(k=0;k<(SeqALength+1);k++)
				{
					sum+=abs(x-k)*PathMatrix[y][k].down;
					sum+=abs(x-k)*PathMatrix[y][k].diagonal;
				}
				for(k=0;k<(SeqBLength+1);k++)
				{
					sum+=abs(y-k)*PathMatrix[k][x].right;
					sum+=abs(y-k)*PathMatrix[k][x].diagonal;
				}
				conf=sum/((double)j*2);
			}
		}
		out2<<"  "<<OutputAlignment1[0].SeqA[i]<<"  "<<OutputAlignment1[0].SeqB[i]<<"  "<<sum<<"  "<<j<<"  "<<conf<<"\n";
	}
	out2.close();
	ofstream out9("confidence1.dat");
	x=0,y=0,k=0;
	for(int i=0;i<OutputAlignment1[0].SeqA.length();i++)
	{
		if(OutputAlignment1[0].SeqA[i]=='-')
		{
			y++;
		}
		else
		{
			if(OutputAlignment1[0].SeqB[i]=='-')
			{
				x++;
			}
			else
			{
				x++;
				y++;
			}
		}
		double sum=0;
		/*if(x-OFFSET<0)
			k=0;
		else
			k=x-OFFSET;*/
		for(k=0;k<(SeqALength+1);k++)
		{
			if(abs(x-k)!=0)
			{
				sum+=10/(double)abs(x-k)*PathMatrix[y][k].down;
				sum+=10/(double)abs(x-k)*PathMatrix[y][k].diagonal;
			}
			else
			{
				sum+=20*PathMatrix[y][k].diagonal;
				sum+=20*PathMatrix[y][k].down;
			}
		}
		/*if(y-OFFSET<0)
			k=0;
		else
			k=y-OFFSET;*/
		for(k=0;k<(SeqBLength+1);k++)
		{
			if(abs(y-k)!=0)
			{
				sum+=10/(double)abs(y-k)*PathMatrix[k][x].right;
				sum+=10/(double)abs(y-k)*PathMatrix[k][x].diagonal;
			}
			else
			{
				sum+=20*PathMatrix[k][x].diagonal;
				sum+=20*PathMatrix[k][x].right;
			}
		}
		//k--;
		//sum=sum-PathMatrix[y][x];
		double conf=sum/((double)j*2);
		out9<<"  "<<OutputAlignment1[0].SeqA[i]<<"  "<<OutputAlignment1[0].SeqB[i]<<"  "<<sum<<"  "<<j<<"  "<<conf<<"\n";
	}
	out9.close();
/*	strcpy(filename, "");
	strcat(filename, "confidence2-");
	strcat(filename, name);
	strcat(filename, ".dat");*/
	ofstream out11("confidence2.dat");
	for(int l=0;l<j;l++)
	{
   	double conf=0;
   	int x=0,y=0,k=0;
   	for(int i=0;i<OutputAlignment1[l].SeqA.length();i++)
   	{
   		if(OutputAlignment1[l].SeqA[i]=='-')
   		{
   			y++;
			double sum=0;
   			for(k=0;k<(SeqALength+1);k++)
   			{
   				sum+=abs(x-k)*PathMatrix[y][k].down;
   				sum+=abs(x-k)*PathMatrix[y][k].diagonal;
   			}
			conf+=sum/((double)j);
   		}
   		else
   		{
   			if(OutputAlignment1[l].SeqB[i]=='-')
   			{
   				x++;
				double sum=0;
    				for(k=0;k<(SeqBLength+1);k++)
   				{
   					sum+=abs(y-k)*PathMatrix[k][x].right;
   					sum+=abs(y-k)*PathMatrix[k][x].diagonal;
   				}
 		  		conf+=sum/((double)j);
   			}
   			else
   			{
   				x++;
   				y++;
				double sum=0;
   				for(k=0;k<(SeqALength+1);k++)
   				{
   					sum+=abs(x-k)*PathMatrix[y][k].down;
   					sum+=abs(x-k)*PathMatrix[y][k].diagonal;
   				}
   				for(k=0;k<(SeqBLength+1);k++)
   				{
   					sum+=abs(y-k)*PathMatrix[k][x].right;
   					sum+=abs(y-k)*PathMatrix[k][x].diagonal;
   				}
   				conf+=(sum/((double)j*2));
   			}
   		}

   	}
   	double alnconf=conf/OutputAlignment1[l].SeqA.length();
   	out11<<alnconf<<"\n";
  }
  out11.close();
	ofstream out10("confidence3.dat");
	int l=OutputAlignment1[0].SeqA.length();
	int x0[l];
	int y0[l];
	x=0, y=0;
	for(int i=0;i<l;i++)
	{
		if(OutputAlignment1[0].SeqA[i]=='-')
		{
			y++;
		}
		else
		{
			if(OutputAlignment1[0].SeqB[i]=='-')
			{
				x++;
			}
			else
			{
				x++;
				y++;
			}
		}
		x0[i]=x;
		y0[i]=y;
	}
	for(int n=1;n<j;n++)
	{
		l=OutputAlignment1[n].SeqA.length();
		double sum=0;
		x=0, y=0;
		for(int i=0;i<l;i++)
		{
			if(OutputAlignment1[n].SeqA[i]=='-')
			{
				y++;
			}
			else
			{
				if(OutputAlignment1[n].SeqB[i]=='-')
				{
					x++;
				}
				else
				{
					x++;
					y++;
				}
			}
			int k=0;
			while(y0[k]!=y)
			{
				k++;
			}
			double absx=(double)abs(x-x0[k]);
			k=0;
			while(x0[k]!=x)
			{
				k++;
			}
			double absy=(double)abs(y-y0[k]);
			sum+=(absx+absy)/2;
		}
		out10<<(sum/l)<<"\n";
	}
	out10.close();
	if(plot_flag!=0)
	{
		ofstream out3("path.eps");
		out3<<"%!PS-Adobe-3.0 EPSF-3.0\n";
		out3<<"%%BoundingBox: 0 0 "<<3*SeqALength<<" "<<3*SeqBLength<<"\n";
		out3<<"1 setlinewidth\n";
		out3<<"0 0 moveto\n";
		out3<<"0 "<<3*SeqBLength<<" lineto\n";
		out3<<3*SeqALength<<" "<<3*SeqBLength<<" lineto\n";
		out3<<3*SeqALength<<" 0 lineto\n";
		out3<<"0 0 lineto\n";
		for(int i=0;i<j;i++)
		{
			int x=0,y=0;
			for(int k=0;k<OutputAlignment1[i].SeqA.length();k++)
			{
				if(OutputAlignment1[i].SeqA[k]=='-')
				{
					y++;
				}
				else
				{
					if(OutputAlignment1[i].SeqB[k]=='-')
					{
						x++;
					}
					else
					{
						x++;
						y++;
					}
				}
				out3<<3*x<<" "<<3*y<<" lineto\n";
			}
			out3<<"0 0 moveto\n";
		}
		out3<<"closepath\n";
		out3<<"stroke\n";
		out3.close();
	}	
	ofstream out8("sw.dat");
	int m=0,n=0,flag=0;
	for(int i=0;i<OutputAlignment1[0].SeqA.length();i++)
	{
		if((OutputAlignment1[0].SeqA[i]=='-')||(OutputAlignment1[0].SeqB[i]=='-'))
		{
			if(flag==0)
			{
				flag=1;
				out8<<OPEN_PENALTY<<endl;
			}
			else
			{
				out8<<AFFINE_PENALTY<<endl;
			}
		}
		else
		{
			if(flag==1)
			{
				flag=0;
			}
			m=GetPosition(OutputAlignment1[0].SeqA[i]);
			n=GetPosition(OutputAlignment1[0].SeqB[i]);
			out8<<ScoreMatrix[m][n]<<endl;
		}
	}
	out8.close();
}

string ReverseString(string a)
{
	string temp;
	int n=a.length();
	for(int i=1;i<=n;i++)
		temp+=a[n-i];
	return temp;
}
	
double GetScore(int a, int b, string SqA, string SqB)
{
  int m=0,n=0;
  m=GetPosition(SqA[b-1]);
  n=GetPosition(SqB[a-1]);
  return ScoreMatrix[m][n];
}
  
int GetPosition(char Base)
{
  switch(Base)
    {
    case 'A':
    case 'a':
      {
	return 0;
	break;
      }
    case 'R':
    case 'r':
      {
	return 1;
	break;
      }
    case 'N':
    case 'n':
      {
	return 2;
	break;
      }
    case 'D':
    case 'd':
      {
	return 3;
	break;
      }
    case 'C':
    case 'c':
      {
	return 4;
	break;
      }
    case 'Q':
    case 'q':
      {
	return 5;
	break;
      }
    case 'E':
    case 'e':
      {
	return 6;
	break;
      }
    case 'G':
    case 'g':
      {
	return 7;
	break;
      }
    case 'H':
    case 'h':
      {
	return 8;
	break;
      }
    case 'I':
    case 'i':
      {
	return 9;
	break;
      }
    case 'L':
    case 'l':
      {
	return 10;
	break;
      }
    case 'K':
    case 'k':
      {
	return 11;
	break;
      }
    case 'M':
    case 'm':
      {
	return 12;
	break;
      }
    case 'F':
    case 'f':
      {
	return 13;
	break;
      }
    case 'P':
    case 'p':
      {
	return 14;
	break;
      }
    case 'S':
    case 's':
      {
	return 15;
	break;
      }
    case 'T':
    case 't':
      {
	return 16;
	break;
      }
    case 'W':
    case 'w':
      {
	return 17;
	break;
      }
    case 'Y':
    case 'y':
      {
	return 18;
	break;
      }
    case 'V':
    case 'v':
      {
	return 19;
	break;
      }
    }
}

void ReadMatrix(char * MatrixName)
{
	char temp[2000];
	char buffer[3];
	ifstream matrix(MatrixName, ios::in);
	for(int i=0;i<8;i++)
		matrix.getline(temp, 2000);
	for(int i=0;i<20;i++)
	{
		matrix.getline(temp, 2000);
		istringstream line(temp);
		int j=0;
		while(line>>ScoreMatrix[i][j])
		{
			ScoreMatrix[j][i]=ScoreMatrix[i][j];
			j++;
		}
	}
	/*matrix>>ws;
	for(int i=0;i<20;i++)
	{
		for(int j=0;j<=i;j++)
		{
			matrix.get(buffer, 4);
			sscanf(buffer,"%f",&ScoreMatrix[i][j]);
			ScoreMatrix[j][i]=ScoreMatrix[i][j];
			matrix>>ws;
		}
	}*/
}

void ReadQuerySecondaryStructure(char* Structure,int* StoreString)
{
	char temp[MAXLENGTH];
	int index=0;
	ifstream QSecStruc(Structure, ios::in);
	if(QSecStruc.good())
	{
		QSecStruc.getline(temp,MAXLENGTH);
		QSecStruc.getline(temp,MAXLENGTH);
		QSecStruc.getline(temp,MAXLENGTH);
		int i=0;
		if(strlen(temp)==SeqALength)
		{
			while(temp[i])
			{
				if(temp[i]=='E')
				{
					StoreString[index]=BETA_SHEET;
				}
				if(temp[i]=='H')
				{
					StoreString[index]=ALPHA_HELIX;
				}
				if(temp[i]=='C')
				{
					StoreString[index]=COIL;
				}
				index++;
				i++;
			}
		}
		else
		{
			cout<<"The length of query secondary structure doesn't match the sequence length!\n";
			exit(1);
		}
	}
	else
	{
		cout<<"The query secondary structure file may not exist!\n";
		exit(2);
	}		
}

void ReadTemplateSecondaryStructure(char* Structure,int* StoreString)
{
	char temp[200];
	int index, filling;
	int space=0;
	ifstream TSecStruc(Structure, ios::in);
	for(int i=0;i<24;i++)
		TSecStruc.getline(temp,200);
	for(int i=0;i<SeqBLength;i++)
	{
		StoreString[i]=COIL;
	}
	int i=0;
	while(TSecStruc.getline(temp,200))
	{
		if(temp[16]=='!')
		{
			space++;
		}
		if(temp[16]=='E')
		{
			sscanf(temp,"%d", &index);
			StoreString[index-space-1]=BETA_SHEET;
		}
		if(temp[16]=='H')
		{
			sscanf(temp,"%d", &index);
			StoreString[index-space-1]=ALPHA_HELIX;
		}
		i++;
	}
/*	if(i != SeqBLength)
	{
		cout<<"The length of template secondary structure doesn't match the sequence length!\n";
		exit(3);
	}*/		
}

string ReadSequenceFile(char* TemplateSequence)
{
	char temp[2000];
	string tempseq;
	ifstream TempSeq(TemplateSequence, ios::in);
	TempSeq.getline(temp, 2000);
	while(TempSeq.getline(temp, 2000))
		tempseq+=temp;
	return tempseq;
}

void ReadRealAlignment(char* RealAlignment, struct realalignment* StoreString)
{
	char temp[500]="";
	char tempchain[500]="";
	int off1=-1;
	int off2=-1;
	int tempoff;
	char chain1[2*MAXLENGTH]="";
	char chain2[2*MAXLENGTH]="";
	ifstream Real(RealAlignment, ios::in);
	for(int i=0;i<10;i++)
		Real.getline(temp, 500);
	while(Real.getline(temp, 500))
	{
		if(sscanf(temp,"Chain 1: %d%s", &tempoff, tempchain)==2)
		{
			strcat(chain1,tempchain);
			if(off1==-1)
			{
				off1=tempoff;
			}
		}
		if(sscanf(temp,"Chain 2: %d%s", &tempoff, tempchain)==2)
		{
			strcat(chain2,tempchain);
			if(off2==-1)
			{
				off2=tempoff;
			}
		}
		if(sscanf(temp,"     X2 = %s", tempchain)==1)
		{
			break;
		} 
	}
	StoreString[0].x=off1;
	StoreString[0].y=off2;
	int i=1;
	while(chain1[i])
	{
		if(chain1[i-1]=='-')
		{
			off2++;
			StoreString[i].x=off1;
			StoreString[i].y=off2;
		}
		else
		{
			if(chain2[i-1]=='-')
			{
				off1++;
				StoreString[i].x=off1;
				StoreString[i].y=off2;
			}
			else
			{
				off1++;
				off2++;
				StoreString[i].x=off1;
				StoreString[i].y=off2;
			}
		}
		i++;
		RealALength++;
	}
}

float Distance(struct realalignment* StoreString, struct outputalignment Pred)
{
	struct realalignment pred[2*MAXLENGTH];
	float disx=0,disy=0,dis=0;
	int i=1;
	int x=0;
	int y=0;
	pred[0].x=0;
	pred[0].y=0;
	for(int i=0;i<2*MAXLENGTH;i++)
		DisSerie[i]=0;
	while(Pred.SeqA[i-1])
	{
		if(Pred.SeqA[i-1]=='-')
		y++;
		else
		{
			if(Pred.SeqB[i-1]=='-')
			x++;
			else
			{
				x++;
				y++;
			}
		}
		pred[i].x=x;
		pred[i].y=y;
		i++;
	}
	for(int i=0;i<RealALength;i++)
	{
		int j=0;
		x=StoreString[i].x;
		for(j=0;j<Pred.SeqA.length();j++)
		{
			if(pred[j].x==x)
				break;
		}
		disy=(float)abs(pred[j].y-StoreString[i].y);
		y=StoreString[i].y;
		for(j=0;j<Pred.SeqA.length();j++)
		{
			if(pred[j].y==y)
				break;
		}
		disx=(float)abs(pred[j].x-StoreString[i].x);
		dis+=(disx+disy)/2;
		DisSerie[i]=(disx+disy)/2;
	}
	return dis/RealALength;
}

float Distance1(struct realalignment* StoreString, struct outputalignment Pred)
{
	struct realalignment pred[2*MAXLENGTH];
	float disx=0,disy=0;
	int i=1;
	int x=0;
	int y=0;
	pred[0].x=0;
	pred[0].y=0;
	for(int i=0;i<2*MAXLENGTH;i++)
		DisSerie[i]=0;
	while(Pred.SeqA[i])
	{
		if(Pred.SeqA[i-1]=='-')
		y++;
		else
		{
			if(Pred.SeqB[i-1]=='-')
			x++;
			else
			{
				x++;
				y++;
			}
		}
		pred[i].x=x;
		pred[i].y=y;
		i++;
	}
	for(int i=0;i<Pred.SeqA.length();i++)
	{
		int j=0;
		x=pred[i].x;
		y=pred[i].y;
		if((x<StoreString[0].x)||(x>StoreString[RealALength-1].x)||(y<StoreString[0].y)||(y>StoreString[RealALength-1].y))
		{
			DisSerie1[i]=-1;
		}
		else
		{
			for(j=0;j<RealALength;j++)
			{
				if(StoreString[j].x==x)
					break;
			}
			disy=(float)abs(pred[i].y-StoreString[j].y);
			for(j=0;j<RealALength;j++)
			{
				if(StoreString[j].y==y)
					break;
			}
			disx=(float)abs(pred[i].x-StoreString[j].x);
			DisSerie1[i]=(disx+disy)/2;
		}
		/*for(j=0;j<Pred.SeqA.length();j++)
		{
			if(pred[j].x==x)
				break;
		}
		disy=(float)abs(pred[j].y-StoreString[i].y);
		y=StoreString[i].y;
		for(j=0;j<Pred.SeqA.length();j++)
		{
			if(pred[j].y==y)
				break;
		}
		disx=(float)abs(pred[j].x-StoreString[i].x);
		dis+=(disx+disy)/2;
		DisSerie[i]=(disx+disy)/2;*/
	}
	//return dis/RealALength;
}

double GetScore1(int a, int b, string SqA, string SqB)
{
	double temp1=0, temp2=0, n1=0, n2=0;
	for(int i=0;i<20;i++)
	{
		temp1+=(log(1-fa1[i][b-1]/20)/-0.051293294)*qf2[i][a-1];
		temp2+=(log(1-fa2[i][a-1]/20)/-0.051293294)*qf1[i][b-1];
		n1+=log(1-fa1[i][b-1]/20)/-0.051293294;
		n2+=log(1-fa2[i][a-1]/20)/-0.051293294;
	}
	double s=(n2-1)/(n1+n2-2)*temp1+(n1-1)/(n1+n2-2)*temp2;
	return s;
}

double GetScore2(int a, int b, string SqA, string SqB)
{
	double s=(GetScore1(a,b,SqA,SqB)-avg)/dev-0.12;
	return s;
}



int ReadProfile(char * Profile, double fa[20][MAXLENGTH], double Nm[20][MAXLENGTH], double qf[20][MAXLENGTH])
{
	char tmpline[20*MAXLENGTH];
	int off;
	int i;
	ifstream TempProfile(Profile, ios::in);
	TempProfile.getline(tmpline,20*MAXLENGTH);
	sscanf(tmpline,"%d",&off);
	if(off!=1)
	{
		cout<<"Profile "<<Profile<<" doesn't start from 1!!!";
		exit(1);
	}
	else
	{
		for(int j=0;j<20;j++)
		{
			i=0;
			TempProfile.getline(tmpline, 20*MAXLENGTH);
			istringstream line(tmpline);
			while(line>>fa[j][i])
			{
				i++;
			}
		}
		TempProfile.getline(tmpline,20*MAXLENGTH);
		for(int j=0;j<20;j++)
		{
			i=0;
			TempProfile.getline(tmpline,20*MAXLENGTH);
			istringstream line(tmpline);
			while(line>>Nm[j][i])
			{
				i++;
			}
		}
		TempProfile.getline(tmpline,20*MAXLENGTH);
		for(int j=0;j<20;j++)
		{
			i=0;
			TempProfile.getline(tmpline,20*MAXLENGTH);
			istringstream line(tmpline);
			while(line>>qf[j][i])
			{
				i++;
			}
		}
	}
	return i;
}
	
void ReverseProfile(double a[20][MAXLENGTH],int length)
{
	double temp;
	for(int i=0;i<length/2;i++)
	{
		for(int j=0;j<20;j++)
		{
			temp=a[j][length-1-i];
			a[j][length-1-i]=a[j][i];
			a[j][i]=temp;
		}
	}
}

void ReverseSS(int *a,int length)
{
	int temp;
	for(int i=0;i<length/2;i++)
	{
		temp=a[length-1-i];
		a[length-1-i]=a[i];
		a[i]=temp;
	}
}
